package com.eclipsesource.json;

import java.io.ObjectInputStream;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
































































public class JsonObject
  extends JsonValue
  implements Iterable
{
  private final List names;
  private final List values;
  private transient JsonObject.HashIndexTable table;
  
  public JsonObject()
  {
    names = new ArrayList();
    values = new ArrayList();
    table = new JsonObject.HashIndexTable();
  }
  





  public JsonObject(JsonObject paramJsonObject)
  {
    this(paramJsonObject, false);
  }
  
  private JsonObject(JsonObject paramJsonObject, boolean paramBoolean) {
    if (paramJsonObject == null) {
      throw new NullPointerException("object is null");
    }
    if (paramBoolean) {
      names = Collections.unmodifiableList(names);
      values = Collections.unmodifiableList(values);
    } else {
      names = new ArrayList(names);
      values = new ArrayList(values);
    }
    table = new JsonObject.HashIndexTable();
    updateHashIndex();
  }
  


















  @Deprecated
  public static JsonObject readFrom(Reader paramReader)
  {
    return JsonValue.readFrom(paramReader).asObject();
  }
  











  @Deprecated
  public static JsonObject readFrom(String paramString)
  {
    return JsonValue.readFrom(paramString).asObject();
  }
  












  public static JsonObject unmodifiableObject(JsonObject paramJsonObject)
  {
    return new JsonObject(paramJsonObject, true);
  }
  

















  public JsonObject add(String paramString, int paramInt)
  {
    add(paramString, Json.value(paramInt));
    return this;
  }
  

















  public JsonObject add(String paramString, long paramLong)
  {
    add(paramString, Json.value(paramLong));
    return this;
  }
  

















  public JsonObject add(String paramString, float paramFloat)
  {
    add(paramString, Json.value(paramFloat));
    return this;
  }
  

















  public JsonObject add(String paramString, double paramDouble)
  {
    add(paramString, Json.value(paramDouble));
    return this;
  }
  

















  public JsonObject add(String paramString, boolean paramBoolean)
  {
    add(paramString, Json.value(paramBoolean));
    return this;
  }
  

















  public JsonObject add(String paramString1, String paramString2)
  {
    add(paramString1, Json.value(paramString2));
    return this;
  }
  

















  public JsonObject add(String paramString, JsonValue paramJsonValue)
  {
    if (paramString == null) {
      throw new NullPointerException("name is null");
    }
    if (paramJsonValue == null) {
      throw new NullPointerException("value is null");
    }
    table.add(paramString, names.size());
    names.add(paramString);
    values.add(paramJsonValue);
    return this;
  }
  
















  public JsonObject set(String paramString, int paramInt)
  {
    set(paramString, Json.value(paramInt));
    return this;
  }
  
















  public JsonObject set(String paramString, long paramLong)
  {
    set(paramString, Json.value(paramLong));
    return this;
  }
  
















  public JsonObject set(String paramString, float paramFloat)
  {
    set(paramString, Json.value(paramFloat));
    return this;
  }
  
















  public JsonObject set(String paramString, double paramDouble)
  {
    set(paramString, Json.value(paramDouble));
    return this;
  }
  
















  public JsonObject set(String paramString, boolean paramBoolean)
  {
    set(paramString, Json.value(paramBoolean));
    return this;
  }
  
















  public JsonObject set(String paramString1, String paramString2)
  {
    set(paramString1, Json.value(paramString2));
    return this;
  }
  















  public JsonObject set(String paramString, JsonValue paramJsonValue)
  {
    if (paramString == null) {
      throw new NullPointerException("name is null");
    }
    if (paramJsonValue == null) {
      throw new NullPointerException("value is null");
    }
    int i = indexOf(paramString);
    if (i != -1) {
      values.set(i, paramJsonValue);
    } else {
      table.add(paramString, names.size());
      names.add(paramString);
      values.add(paramJsonValue);
    }
    return this;
  }
  








  public JsonObject remove(String paramString)
  {
    if (paramString == null) {
      throw new NullPointerException("name is null");
    }
    int i = indexOf(paramString);
    if (i != -1) {
      table.remove(i);
      names.remove(i);
      values.remove(i);
    }
    return this;
  }
  








  public JsonObject merge(JsonObject paramJsonObject)
  {
    if (paramJsonObject == null) {
      throw new NullPointerException("object is null");
    }
    for (JsonObject.Member localMember : paramJsonObject) {
      set(JsonObject.Member.access$000(localMember), JsonObject.Member.access$100(localMember));
    }
    return this;
  }
  








  public JsonValue get(String paramString)
  {
    if (paramString == null) {
      throw new NullPointerException("name is null");
    }
    int i = indexOf(paramString);
    return i != -1 ? (JsonValue)values.get(i) : null;
  }
  













  public int getInt(String paramString, int paramInt)
  {
    JsonValue localJsonValue = get(paramString);
    return localJsonValue != null ? localJsonValue.asInt() : paramInt;
  }
  













  public long getLong(String paramString, long paramLong)
  {
    JsonValue localJsonValue = get(paramString);
    return localJsonValue != null ? localJsonValue.asLong() : paramLong;
  }
  













  public float getFloat(String paramString, float paramFloat)
  {
    JsonValue localJsonValue = get(paramString);
    return localJsonValue != null ? localJsonValue.asFloat() : paramFloat;
  }
  













  public double getDouble(String paramString, double paramDouble)
  {
    JsonValue localJsonValue = get(paramString);
    return localJsonValue != null ? localJsonValue.asDouble() : paramDouble;
  }
  













  public boolean getBoolean(String paramString, boolean paramBoolean)
  {
    JsonValue localJsonValue = get(paramString);
    return localJsonValue != null ? localJsonValue.asBoolean() : paramBoolean;
  }
  












  public String getString(String paramString1, String paramString2)
  {
    JsonValue localJsonValue = get(paramString1);
    return localJsonValue != null ? localJsonValue.asString() : paramString2;
  }
  




  public int size()
  {
    return names.size();
  }
  




  public boolean isEmpty()
  {
    return names.isEmpty();
  }
  






  public List names()
  {
    return Collections.unmodifiableList(names);
  }
  





  public Iterator iterator()
  {
    Iterator localIterator1 = names.iterator();
    Iterator localIterator2 = values.iterator();
    return new JsonObject.1(this, localIterator1, localIterator2);
  }
  
















  void write(JsonWriter paramJsonWriter)
  {
    paramJsonWriter.writeObjectOpen();
    Iterator localIterator1 = names.iterator();
    Iterator localIterator2 = values.iterator();
    int i = 1;
    while (localIterator1.hasNext()) {
      if (i == 0) {
        paramJsonWriter.writeObjectSeparator();
      }
      paramJsonWriter.writeMemberName((String)localIterator1.next());
      paramJsonWriter.writeMemberSeparator();
      ((JsonValue)localIterator2.next()).write(paramJsonWriter);
      i = 0;
    }
    paramJsonWriter.writeObjectClose();
  }
  
  public boolean isObject()
  {
    return true;
  }
  
  public JsonObject asObject()
  {
    return this;
  }
  
  public int hashCode()
  {
    int i = 1;
    i = 31 * i + names.hashCode();
    i = 31 * i + values.hashCode();
    return i;
  }
  
  public boolean equals(Object paramObject)
  {
    if (this == paramObject) {
      return true;
    }
    if (paramObject == null) {
      return false;
    }
    if (getClass() != paramObject.getClass()) {
      return false;
    }
    JsonObject localJsonObject = (JsonObject)paramObject;
    return (names.equals(names)) && (values.equals(values));
  }
  
  int indexOf(String paramString) {
    int i = table.get(paramString);
    if ((i != -1) && (paramString.equals(names.get(i)))) {
      return i;
    }
    return names.lastIndexOf(paramString);
  }
  

  private synchronized void readObject(ObjectInputStream paramObjectInputStream)
  {
    paramObjectInputStream.defaultReadObject();
    table = new JsonObject.HashIndexTable();
    updateHashIndex();
  }
  
  private void updateHashIndex() {
    int i = names.size();
    for (int j = 0; j < i; j++) {
      table.add((String)names.get(j), j);
    }
  }
}
