package com.eclipsesource.json;

import java.io.Reader;
import java.io.StringReader;

































class JsonParser
{
  private static final int MIN_BUFFER_SIZE = 10;
  private static final int DEFAULT_BUFFER_SIZE = 1024;
  private final Reader reader;
  private final char[] buffer;
  private int bufferOffset;
  private int index;
  private int fill;
  private int line;
  private int lineOffset;
  private int current;
  private StringBuilder captureBuffer;
  private int captureStart;
  
  JsonParser(String paramString)
  {
    this(new StringReader(paramString), 
      Math.max(10, Math.min(1024, paramString.length())));
  }
  
  JsonParser(Reader paramReader) {
    this(paramReader, 1024);
  }
  
  JsonParser(Reader paramReader, int paramInt) {
    reader = paramReader;
    buffer = new char[paramInt];
    line = 1;
    captureStart = -1;
  }
  
  JsonValue parse() {
    read();
    skipWhiteSpace();
    JsonValue localJsonValue = readValue();
    skipWhiteSpace();
    if (!isEndOfText()) {
      throw error("Unexpected character");
    }
    return localJsonValue;
  }
  
  private JsonValue readValue() {
    switch (current) {
    case 110: 
      return readNull();
    case 116: 
      return readTrue();
    case 102: 
      return readFalse();
    case 34: 
      return readString();
    case 91: 
      return readArray();
    case 123: 
      return readObject();
    case 45: 
    case 48: 
    case 49: 
    case 50: 
    case 51: 
    case 52: 
    case 53: 
    case 54: 
    case 55: 
    case 56: 
    case 57: 
      return readNumber();
    }
    throw expected("value");
  }
  
  private JsonArray readArray()
  {
    read();
    JsonArray localJsonArray = new JsonArray();
    skipWhiteSpace();
    if (readChar(']')) {
      return localJsonArray;
    }
    do {
      skipWhiteSpace();
      localJsonArray.add(readValue());
      skipWhiteSpace();
    } while (readChar(','));
    if (!readChar(']')) {
      throw expected("',' or ']'");
    }
    return localJsonArray;
  }
  
  private JsonObject readObject() {
    read();
    JsonObject localJsonObject = new JsonObject();
    skipWhiteSpace();
    if (readChar('}')) {
      return localJsonObject;
    }
    do {
      skipWhiteSpace();
      String str = readName();
      skipWhiteSpace();
      if (!readChar(':')) {
        throw expected("':'");
      }
      skipWhiteSpace();
      localJsonObject.add(str, readValue());
      skipWhiteSpace();
    } while (readChar(','));
    if (!readChar('}')) {
      throw expected("',' or '}'");
    }
    return localJsonObject;
  }
  
  private String readName() {
    if (current != 34) {
      throw expected("name");
    }
    return readStringInternal();
  }
  
  private JsonValue readNull() {
    read();
    readRequiredChar('u');
    readRequiredChar('l');
    readRequiredChar('l');
    return Json.NULL;
  }
  
  private JsonValue readTrue() {
    read();
    readRequiredChar('r');
    readRequiredChar('u');
    readRequiredChar('e');
    return Json.TRUE;
  }
  
  private JsonValue readFalse() {
    read();
    readRequiredChar('a');
    readRequiredChar('l');
    readRequiredChar('s');
    readRequiredChar('e');
    return Json.FALSE;
  }
  
  private void readRequiredChar(char paramChar) {
    if (!readChar(paramChar)) {
      throw expected("'" + paramChar + "'");
    }
  }
  
  private JsonValue readString() {
    return new JsonString(readStringInternal());
  }
  
  private String readStringInternal() {
    read();
    startCapture();
    while (current != 34) {
      if (current == 92) {
        pauseCapture();
        readEscape();
        startCapture();
      } else { if (current < 32) {
          throw expected("valid string character");
        }
        read();
      }
    }
    String str = endCapture();
    read();
    return str;
  }
  
  private void readEscape() {
    read();
    switch (current) {
    case 34: 
    case 47: 
    case 92: 
      captureBuffer.append((char)current);
      break;
    case 98: 
      captureBuffer.append('\b');
      break;
    case 102: 
      captureBuffer.append('\f');
      break;
    case 110: 
      captureBuffer.append('\n');
      break;
    case 114: 
      captureBuffer.append('\r');
      break;
    case 116: 
      captureBuffer.append('\t');
      break;
    case 117: 
      char[] arrayOfChar = new char[4];
      for (int i = 0; i < 4; i++) {
        read();
        if (!isHexDigit()) {
          throw expected("hexadecimal digit");
        }
        arrayOfChar[i] = ((char)current);
      }
      captureBuffer.append((char)Integer.parseInt(new String(arrayOfChar), 16));
      break;
    default: 
      throw expected("valid escape sequence");
    }
    read();
  }
  
  private JsonValue readNumber() {
    startCapture();
    readChar('-');
    int i = current;
    if (!readDigit()) {
      throw expected("digit");
    }
    while ((i != 48) && 
      (readDigit())) {}
    

    readFraction();
    readExponent();
    return new JsonNumber(endCapture());
  }
  
  private boolean readFraction() {
    if (!readChar('.')) {
      return false;
    }
    if (!readDigit()) {
      throw expected("digit");
    }
    while (readDigit()) {}
    
    return true;
  }
  
  private boolean readExponent() {
    if ((!readChar('e')) && (!readChar('E'))) {
      return false;
    }
    if (!readChar('+')) {
      readChar('-');
    }
    if (!readDigit()) {
      throw expected("digit");
    }
    while (readDigit()) {}
    
    return true;
  }
  
  private boolean readChar(char paramChar) {
    if (current != paramChar) {
      return false;
    }
    read();
    return true;
  }
  
  private boolean readDigit() {
    if (!isDigit()) {
      return false;
    }
    read();
    return true;
  }
  
  private void skipWhiteSpace() {
    while (isWhiteSpace()) {
      read();
    }
  }
  
  private void read() {
    if (index == fill) {
      if (captureStart != -1) {
        captureBuffer.append(buffer, captureStart, fill - captureStart);
        captureStart = 0;
      }
      bufferOffset += fill;
      fill = reader.read(buffer, 0, buffer.length);
      index = 0;
      if (fill == -1) {
        current = -1;
        return;
      }
    }
    if (current == 10) {
      line += 1;
      lineOffset = (bufferOffset + index);
    }
    current = buffer[(index++)];
  }
  
  private void startCapture() {
    if (captureBuffer == null) {
      captureBuffer = new StringBuilder();
    }
    captureStart = (index - 1);
  }
  
  private void pauseCapture() {
    int i = current == -1 ? index : index - 1;
    captureBuffer.append(buffer, captureStart, i - captureStart);
    captureStart = -1;
  }
  
  private String endCapture() {
    int i = current == -1 ? index : index - 1;
    String str;
    if (captureBuffer.length() > 0) {
      captureBuffer.append(buffer, captureStart, i - captureStart);
      str = captureBuffer.toString();
      captureBuffer.setLength(0);
    } else {
      str = new String(buffer, captureStart, i - captureStart);
    }
    captureStart = -1;
    return str;
  }
  
  private ParseException expected(String paramString) {
    if (isEndOfText()) {
      return error("Unexpected end of input");
    }
    return error("Expected " + paramString);
  }
  
  private ParseException error(String paramString) {
    int i = bufferOffset + index;
    int j = i - lineOffset;
    int k = isEndOfText() ? i : i - 1;
    return new ParseException(paramString, k, line, j - 1);
  }
  
  private boolean isWhiteSpace() {
    return (current == 32) || (current == 9) || (current == 10) || (current == 13);
  }
  
  private boolean isDigit() {
    return (current >= 48) && (current <= 57);
  }
  
  private boolean isHexDigit() {
    return ((current >= 48) && (current <= 57)) || ((current >= 97) && (current <= 102)) || ((current >= 65) && (current <= 70));
  }
  

  private boolean isEndOfText()
  {
    return current == -1;
  }
}
