package com.eclipsesource.json;

import java.io.Writer;

























class JsonWriter
{
  private static final int CONTROL_CHARACTERS_END = 31;
  private static final char[] QUOT_CHARS = { '\\', '"' };
  private static final char[] BS_CHARS = { '\\', '\\' };
  private static final char[] LF_CHARS = { '\\', 'n' };
  private static final char[] CR_CHARS = { '\\', 'r' };
  private static final char[] TAB_CHARS = { '\\', 't' };
  

  private static final char[] UNICODE_2028_CHARS = { '\\', 'u', '2', '0', '2', '8' };
  private static final char[] UNICODE_2029_CHARS = { '\\', 'u', '2', '0', '2', '9' };
  private static final char[] HEX_DIGITS = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };
  
  protected final Writer writer;
  
  JsonWriter(Writer paramWriter)
  {
    writer = paramWriter;
  }
  
  protected void writeLiteral(String paramString) {
    writer.write(paramString);
  }
  
  protected void writeNumber(String paramString) {
    writer.write(paramString);
  }
  
  protected void writeString(String paramString) {
    writer.write(34);
    writeJsonString(paramString);
    writer.write(34);
  }
  
  protected void writeArrayOpen() {
    writer.write(91);
  }
  
  protected void writeArrayClose() {
    writer.write(93);
  }
  
  protected void writeArraySeparator() {
    writer.write(44);
  }
  
  protected void writeObjectOpen() {
    writer.write(123);
  }
  
  protected void writeObjectClose() {
    writer.write(125);
  }
  
  protected void writeMemberName(String paramString) {
    writer.write(34);
    writeJsonString(paramString);
    writer.write(34);
  }
  
  protected void writeMemberSeparator() {
    writer.write(58);
  }
  
  protected void writeObjectSeparator() {
    writer.write(44);
  }
  
  protected void writeJsonString(String paramString) {
    int i = paramString.length();
    int j = 0;
    for (int k = 0; k < i; k++) {
      char[] arrayOfChar = getReplacementChars(paramString.charAt(k));
      if (arrayOfChar != null) {
        writer.write(paramString, j, k - j);
        writer.write(arrayOfChar);
        j = k + 1;
      }
    }
    writer.write(paramString, j, i - j);
  }
  
  private static char[] getReplacementChars(char paramChar) {
    if (paramChar > '\\') {
      if ((paramChar < ' ') || (paramChar > ' '))
      {
        return null;
      }
      return paramChar == ' ' ? UNICODE_2028_CHARS : UNICODE_2029_CHARS;
    }
    if (paramChar == '\\') {
      return BS_CHARS;
    }
    if (paramChar > '"')
    {
      return null;
    }
    if (paramChar == '"') {
      return QUOT_CHARS;
    }
    if (paramChar > '\037') {
      return null;
    }
    if (paramChar == '\n') {
      return LF_CHARS;
    }
    if (paramChar == '\r') {
      return CR_CHARS;
    }
    if (paramChar == '\t') {
      return TAB_CHARS;
    }
    return new char[] { '\\', 'u', '0', '0', HEX_DIGITS[(paramChar >> '\004' & 0xF)], HEX_DIGITS[(paramChar & 0xF)] };
  }
}
