package com.mojang.util;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.core.Filter;
import org.apache.logging.log4j.core.Layout;
import org.apache.logging.log4j.core.LogEvent;
import org.apache.logging.log4j.core.appender.AbstractAppender;
import org.apache.logging.log4j.core.config.plugins.Plugin;
import org.apache.logging.log4j.core.config.plugins.PluginAttribute;
import org.apache.logging.log4j.core.config.plugins.PluginElement;
import org.apache.logging.log4j.core.layout.PatternLayout;

@Plugin(name="Queue", category="Core", elementType="appender", printObject=true)
public class QueueLogAppender extends AbstractAppender
{
  private static final int MAX_CAPACITY = 250;
  private static final Map QUEUES = new HashMap();
  private static final ReadWriteLock QUEUE_LOCK = new ReentrantReadWriteLock();
  private final BlockingQueue queue;
  
  public QueueLogAppender(String paramString, Filter paramFilter, Layout paramLayout, boolean paramBoolean, BlockingQueue paramBlockingQueue)
  {
    super(paramString, paramFilter, paramLayout, paramBoolean);
    queue = paramBlockingQueue;
  }
  
  public void append(LogEvent paramLogEvent)
  {
    if (queue.size() >= 250) {
      queue.clear();
    }
    queue.add(getLayout().toSerializable(paramLogEvent).toString());
  }
  
  @org.apache.logging.log4j.core.config.plugins.PluginFactory
  public static QueueLogAppender createAppender(@PluginAttribute("name") String paramString1, @PluginAttribute("ignoreExceptions") String paramString2, @PluginElement("Layout") Layout paramLayout, @PluginElement("Filters") Filter paramFilter, @PluginAttribute("target") String paramString3) {
    boolean bool = Boolean.parseBoolean(paramString2);
    
    if (paramString1 == null) {
      LOGGER.error("No name provided for QueueLogAppender");
      return null;
    }
    
    if (paramString3 == null) {
      paramString3 = paramString1;
    }
    
    QUEUE_LOCK.writeLock().lock();
    Object localObject = (BlockingQueue)QUEUES.get(paramString3);
    if (localObject == null) {
      localObject = new LinkedBlockingQueue();
      QUEUES.put(paramString3, localObject);
    }
    QUEUE_LOCK.writeLock().unlock();
    
    if (paramLayout == null) {
      paramLayout = PatternLayout.createLayout(null, null, null, null, null);
    }
    
    return new QueueLogAppender(paramString1, paramFilter, paramLayout, bool, (BlockingQueue)localObject);
  }
  
  public static String getNextLogEvent(String paramString) {
    QUEUE_LOCK.readLock().lock();
    BlockingQueue localBlockingQueue = (BlockingQueue)QUEUES.get(paramString);
    QUEUE_LOCK.readLock().unlock();
    
    if (localBlockingQueue != null) {
      try {
        return (String)localBlockingQueue.take();
      }
      catch (InterruptedException localInterruptedException) {}
    }
    
    return null;
  }
}
